/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "../gwsupport.h"
#include "gwnotebooksearchcallback.h"

#include "gwnotebooksearch.h"
#include "gwstatusbar.h"
#include "gwstatusbarcallback.h"
#include "gwmsgbox.h"

#include "data/gwdata.h"

#include "../gwsearchengine.h"
#include "../gwguimanager.h"


gboolean gw_notebook_search_column_sort ( GtkCList *clist, gint column, gpointer data)
{
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( clist != NULL )
	{
		if ( column == clist->sort_column )
		{
			if ( clist->sort_type == GTK_SORT_ASCENDING )
			{
				clist->sort_type = GTK_SORT_DESCENDING;
			}
			else
			{
				clist->sort_type = GTK_SORT_ASCENDING;
			}
		}
		else
		{
			gtk_clist_set_sort_column ( clist, column);
		}

		gtk_clist_sort ( clist);

		result = TRUE;
	}

	return result;
}


gboolean gw_notebook_search_search_button_click ( GtkWidget *bt, GtkWindow *w)
{
	gboolean result = FALSE;
	struct user_quick_search *user_qs = NULL;
	gulong founded = 0;
	gchar *msg = NULL;
	GtkWindow *window = NULL;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( w != NULL )
	{
		if ( gw_gui_manager_main_interface_get_tree_root ( ) != NULL )
		{
			/* Allocate user_quick_search structure, had to free this structure after */
			user_qs = gw_notebook_search_get_user_quick_search ( w);
			window = gw_gui_manager_main_interface_get_main_window ( );

			gw_status_bar_put_messages ( window, _( "Search in catalog"), _( "Please wait..."));
			gw_gui_manager_main_interface_refresh ( );

			founded = search_engine_search_in_disk ( w, user_qs);
			msg = g_strdup_printf ( _("%lu item(s) founded."), founded);
			gw_status_bar_put_messages ( window, _( "Search result"), msg);
			g_free ( msg);

			/* Free the user_quick_search struture. */
			user_quick_search_free ( user_qs);

			if ( founded == 0 )
			{
				gw_msg_box_create ( w, _( "Search result"), _( "No item matched."));
				gdk_beep ( );
			}

			result = TRUE;
		}
		else
		{
			gw_msg_box_create ( w, _( "Searching..."), _( "Cannot start searching : no catalog loaded"));

			result = FALSE;
		}
	}

	return result;
}


gboolean gw_notebook_search_select_row ( GtkCList *clist, gint line, gint column, GdkEventButton *event, GtkWindow *w)
{
	gboolean result = FALSE;


#ifdef GW_DEBUG_GUI_CALLBACK_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( clist != NULL && w != NULL )
	{
		gw_status_bar_set_catalog_search_page ( w, g_list_length ( clist->selection));

		result = TRUE;
	}

	return result;
}
