/*  GWhere.
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "gwsupport.h"
#include "gwpluginsmanager.h"

#include <gmodule.h>
#include <dirent.h> /* opendir readir closedir*/
#include <string.h> /* strstr */

#include "gwapplicationmanager.h"


GWPluginsManager my_plugins_manager;


gint gw_plugins_manager_init ( )
{
	gint result = -1;
	GModule *module = NULL;
	gchar *module_path = NULL;
	gchar* (*func_get_author)(void);
	gchar* (*func_get_info)(void);
	gchar* (*func_get_help)(void);
	gchar* (*func_get_allowed_ext)(void);
	gchar* (*func_get_descr)(const char*, const char*);
	DIR *plugins_descr_dir = NULL;
	struct dirent *dentry = NULL;
	char *so = NULL;
	gchar **allowed_ext = NULL;
	int i = 0;
	gchar *plugins_dir = NULL;


#if GW_DEBUG_MODE
	gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, NULL);
#endif

	if ( g_module_supported ( ) )
	{
		my_plugins_manager.descr_plugins = g_hash_table_new ( g_str_hash, g_str_equal);
		my_plugins_manager.descr_funcs = g_hash_table_new ( g_str_hash, g_str_equal);
	
#if GW_DEBUG_MODE
		gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, "Starting the scan of %s...", PACKAGE_PLUGINS_DESCRIPTION_DIR);
#endif
		if ( (plugins_dir = gw_package_get_plugins_dir ( )) != NULL )
		{
			if ( (plugins_descr_dir = opendir ( plugins_dir)) != NULL )
			{
				while ( (dentry = readdir ( plugins_descr_dir)) != NULL )
				{
#if GW_DEBUG_MODE
					gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, "Current file is %s.", dentry->d_name);
#endif
	
#if defined ( OS_WIN32)
					if ( (dentry->d_name[0] != '.') && ((so = strrchr ( dentry->d_name, '.')) != NULL) && (strcmp ( so, ".dll") == 0) )
#else
					if ( (dentry->d_name[0] != '.') && (dentry->d_ino > 0) && ((so = strrchr ( dentry->d_name, '.')) != NULL) && (strcmp ( so, ".so") == 0) )
#endif
					{
						module_path = g_strconcat ( PACKAGE_PLUGINS_DESCRIPTION_DIR, G_DIR_SEPARATOR_S, dentry->d_name, NULL);
		
#if GW_DEBUG_MODE
						gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, "Trying to load %s...", module_path);
#endif
	
						module = g_module_open ( module_path, 0);
		
						if ( module != NULL )
						{
							g_module_symbol ( module, "plugin_get_author", (gpointer *)(&func_get_author));
							g_module_symbol ( module, "plugin_get_info", (gpointer *)(&func_get_info));
							g_module_symbol ( module, "plugin_get_help", (gpointer *)(&func_get_help));
							g_module_symbol ( module, "plugin_get_allowed_ext", (gpointer *)(&func_get_allowed_ext));
							g_module_symbol ( module, "plugin_get_descr", (gpointer *)(&func_get_descr));
		
							if ( (&func_get_author!=NULL) && (&func_get_info!=NULL) && (&func_get_help!=NULL) && (&func_get_allowed_ext!=NULL) && (&func_get_descr!=NULL))
							{
#if GW_DEBUG_MODE
								gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, "Module name is %s", g_module_name ( module));
								gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, "Module author is %s", func_get_author ( ));
								gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, "Module info is %s", func_get_info ( ));
								gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, "Module help is %s", func_get_help ( ));
								gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, "Module allowed_ext is %s", func_get_allowed_ext ( ));
#endif
		
								if ( (func_get_allowed_ext ( ) != NULL) && (strlen ( func_get_allowed_ext ( )) > 0) )
								{
									allowed_ext = g_strsplit ( func_get_allowed_ext ( ), "|", -1);
		
									if ( allowed_ext != NULL )
									{
										for ( i = 0; allowed_ext[i] != NULL; i++)
										{
											g_hash_table_insert ( my_plugins_manager.descr_plugins, func_get_allowed_ext ( ), module);
											g_hash_table_insert ( my_plugins_manager.descr_funcs, allowed_ext[i], func_get_descr);
										}
									}
								}
							}
						}
		
						g_free ( module_path);
					}
					else
					{
#if GW_DEBUG_MODE
						gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, "Inode is %d.", dentry->d_ino);
						gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, "Starting character is %c.", dentry->d_name[0]);
						gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, "Extension is %s.", so);
						gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, "Extension is %s.", strrchr ( dentry->d_name, '.'));
#endif
					}
				}
		
				closedir ( plugins_descr_dir);
			}
			else
			{
#if GW_DEBUG_MODE
			gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, "%s not found.", PACKAGE_PLUGINS_DESCRIPTION_DIR);
#endif
			}
	
			g_free ( plugins_dir);
		}

#if GW_DEBUG_MODE
		gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, "Scan of %s is stopped.", PACKAGE_PLUGINS_DESCRIPTION_DIR);
#endif
	}
	else
	{
#if GW_DEBUG_MODE
		gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, "Dynamic modules are not supported!!");
#endif
	}

	return result;
}


gboolean gw_plugins_manager_module_close ( gchar * key, GModule *value, gpointer data)
{
	gboolean result = FALSE;


#if GW_DEBUG_MODE
	gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, NULL);
#endif

	if ( value != NULL )
	{
		g_module_close ( value);
	}

	return result;
}


gint gw_plugins_manager_get_descr_func ( const gchar *ext, gchar* (**func_get_descr)(const char*, const char*))
{
	gint result = -1;


#if GW_DEBUG_MODE
	gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, NULL);
#endif

	if ( my_plugins_manager.descr_funcs != NULL )
	{
		*func_get_descr = g_hash_table_lookup ( my_plugins_manager.descr_funcs, ext);
	}

	return result;
}


gint gw_plugins_manager_exit ( )
{
	gint result = -1;


#if GW_DEBUG_MODE
	gw_application_manager_log_msg ( 0, __FILE__, __LINE__, __PRETTY_FUNCTION__, NULL);
#endif

	if ( my_plugins_manager.catalog_plugins != NULL )
	{
		g_hash_table_foreach_remove ( my_plugins_manager.catalog_plugins, (GHRFunc)gw_plugins_manager_module_close, NULL);
		g_hash_table_destroy ( my_plugins_manager.catalog_plugins);
		my_plugins_manager.catalog_plugins = NULL;
	}

	if ( my_plugins_manager.descr_funcs != NULL )
	{
		g_hash_table_destroy ( my_plugins_manager.descr_funcs);
		my_plugins_manager.descr_funcs = NULL;
	}

	if ( my_plugins_manager.descr_plugins != NULL )
	{
		g_hash_table_foreach_remove ( my_plugins_manager.descr_plugins, (GHRFunc)gw_plugins_manager_module_close, NULL);
		g_hash_table_destroy ( my_plugins_manager.descr_plugins);
		my_plugins_manager.descr_plugins = NULL;
	}

	return result;
}
