/*  GWhere
 *  Copyright (C) 2000  Sbastien LECACHEUR
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "../gwsupport.h"
#include "gwstring.h"

#include <time.h> /* asctime */
#include <string.h> /* strcmp strlen memmove strstr strncmp strerror memcpy */
#include <ctype.h> /* isspace*/
#include <regex.h> /* regcomp regexec */


#ifndef strsep
char *strsep ( char **stringp, const char *delim)
{
	char *res = NULL;


	if ( !stringp || !*stringp || !**stringp )
	{
		return (char*)0;
	}

	res = *stringp;

	while ( **stringp && !strchr ( delim, **stringp) )
	{
		(*stringp)++;
	}

	if(**stringp)
	{
		**stringp = '\0';
		(*stringp)++;
	}

	return res;
}
#endif  /* strsep */


unsigned long int gw_ntohl ( unsigned long int in)
{
	unsigned char a[4], t;


	memcpy ( a, &in, 4);

	t = a[3];
	a[3] = a[0];
	a[0] = t;

	t = a[2];
	a[2] = a[1];
	a[1] = t;

	memcpy ( &in, a, 4);

	return in;
}


gchar * gw_l_byte_to_str_format ( long size )
{
	long double n = size * 1.0;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	return gw_ld_byte_to_str_format ( n);
}


gchar * gw_ul_byte_to_str_format ( unsigned long size )
{
	long double n = size * 1.0;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	return gw_ld_byte_to_str_format ( n);
}


gchar * gw_ui64_byte_to_str_format ( guint64 size )
{
	long double n = size * 1.0;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	return gw_ld_byte_to_str_format ( n);
}


gchar * gw_d_byte_to_str_format ( gdouble size )
{
	long double n = size * 1.0;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	return gw_ld_byte_to_str_format ( n);
}


gchar * gw_ld_byte_to_str_format ( long double n)
{
	long double m = n * 1.0;
	gint i = 0;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	while ( (m = (m / 1024.0)) >= 1 )
	{
		i++;
	}

	switch ( i)
	{
		case 0 :	return g_strdup_printf ( _( "%.0f bytes"), (double)n);

				break;

		case 1 :	return g_strdup_printf ( _( "%.2f Kb"), (double)(n / 1024));

				break;

		case 2 :	return g_strdup_printf ( _( "%.2f Mb"), (double)(n / 1048576));

				break;

		case 3 :	return g_strdup_printf ( _( "%.2f Gb"), (double)(n / 1073741824));

				break;

		case 4 :	m = n / 1073741824;
				return g_strdup_printf ( _( "%.2f Tb"), (double)( m / 1024));

				break;

		case 5 :	m = n / 1073741824;
				return g_strdup_printf ( _( "%.2f Pb"), (double)( m / 1048576));

				break;

		case 6 :	m = n / 1073741824;
				return g_strdup_printf ( _( "%.2f Hb"), (double)( m / 1073741824));

				break;

		case 7 :	m = n / 1073741824;
				m = m / 1073741824;
				return g_strdup_printf ( _( "%.2f Ib"), (double)( m / 1024));

				break;

		default :	m = n / 1073741824;
				m = m / 1073741824;
				return g_strdup_printf ( _( "%.2f Ib"), (double)( m / 1024));

				break;
	}
}


long double gw_str_format_to_ld ( gchar * nb)
{
	long double n;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( nb[strlen ( nb) -2] == ' ')
	{
		/* Size in byte */
		return g_strtod ( nb, NULL);
	}
	else { if ( nb[strlen ( nb) -2] == 'K')
	{
		/* Size in kilo byte */
		n = 1024 * g_strtod ( nb, NULL);

		return n;
	}
	else { if ( nb[strlen ( nb) -2] == 'M')
	{
		/* Size in mega byte */
		n = 1048576 * g_strtod ( nb, NULL);

		return n;
	}
	else { if ( nb[strlen ( nb) -2] == 'G')
	{
		/* Size in giga byte */
		n = 1073741824 * g_strtod ( nb, NULL);

		return n;
	}
	else { if ( nb[strlen ( nb) -2] == 'T')
	{
		/* Size in tera byte */
		n = 1024;
		n = n * 1073741824;
		n = n * g_strtod ( nb, NULL);

		return n;
	}
	else { if ( nb[strlen ( nb) -2] == 'P')
	{
		/* Size in penta byte */
		n = 1048576;
		n = n * 1073741824;
		n = n * g_strtod ( nb, NULL);

		return n;
	}
	else { if ( nb[strlen ( nb) -2] == 'H')
	{
		/* Size in hexa byte */
		n = 1024 * g_strtod ( nb, NULL);

		return n;
	}
	else { if ( nb[strlen ( nb) -2] == 'I')
	{
		/* Size in iota byte */
		n = 1073741824;
		n = n * 1073741824;
		n = n * g_strtod ( nb, NULL);

		return n;
	}}}}}}}}

	return 0;
}


gchar * gw_tm_to_str_format ( struct tm* p)
{
        gchar * date = NULL;


#ifdef GW_DEBUG_TOOLS_COMPONENT
        g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

        if ( p != NULL )
        {
#ifdef __DEPRECATED__
		/* The code has changed to fix untranslated date under win32. */
                date = asctime ( p);

                if ( date != NULL )
                {
                	/* Add this code for unhoped end character on win32. */
                        if ( isprint ( date[strlen ( date)-1]) == 0)
                        {
                                date[strlen ( date)-1]='\0';
                        }
                }
#else
		if ( (date = (gchar*)g_malloc ( sizeof ( gchar) * 20)) != NULL )
		{
			strftime ( date, 20, "%d/%m/%Y %H:%M:%S", p);
		}
#endif
        }

        return date;
}


gchar * gw_str_to_file ( gchar *desc)
{
	gchar **buf = NULL;
	gchar *txt = NULL;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : string=%s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, desc);
#endif

	if ( desc != NULL )
	{
		txt = g_strdup ( desc);

#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : first step %s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, txt);
#endif

		buf = g_strsplit ( txt, "\n", 0);

		if ( buf != NULL )
		{
			g_free ( txt);
			txt = g_strjoinv ( "\\n", buf);
#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : second step %s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, txt);
#endif


			g_strfreev ( buf);
			buf = NULL;

			buf = g_strsplit ( txt, ":", 0);

			if ( buf != NULL )
			{
				g_free ( txt);
				txt = g_strjoinv ( "\\#", buf);

#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : third step %s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, txt);
#endif

				g_strfreev ( buf);
				buf = NULL;
			}
		}
	}

#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : encoded string=%s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, txt);
#endif

	return txt;
}


gchar * gw_file_to_str ( gchar *desc)
{
	gchar **buf = NULL;
	gchar *txt = NULL;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : decode string=%s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, desc);
#endif

	if ( desc != NULL )
	{
		txt = g_strdup ( desc);
		buf = g_strsplit ( txt, "\\n", 0);

		if ( buf != NULL )
		{
			g_free ( txt);
			txt = g_strjoinv ( "\n", buf);

			g_strfreev ( buf);
			buf = NULL;

			buf = g_strsplit ( txt, "\\#", 0);

			if ( buf != NULL )
			{
				g_free ( txt);
				txt = g_strjoinv ( ":", buf);

				g_strfreev ( buf);
				buf = NULL;
			}
		}
	}


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s() : decoded string=%s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, txt);
#endif

	return txt;
}


gchar * gw_str_replace_str ( gchar *patern, gchar *src, gchar *dst)
{
	gchar **fields = NULL;
	gchar *result = NULL;
	gchar *check_end = NULL;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( patern != NULL )
	{
		fields = g_strsplit ( patern, src, 0);
		result = g_strjoinv ( dst, fields);

		if ( strcmp ( src, patern + strlen ( patern) - strlen ( src)) == 0 )
		{
			check_end = result;
			result = g_strconcat ( check_end, dst, NULL);
			g_free ( check_end);
		}
	}

	return result;
}


gchar *gw_str_replace_strv ( gchar *patern, gchar **src, gchar **dst)
{
	gchar *result = NULL;
	gchar *build_patern = NULL;
	gint i = 0;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( patern != NULL )
	{
		while ( ( src[i] != NULL ) || ( dst[i] != NULL ) )
		{
			build_patern = patern;
			patern = gw_str_replace_str ( patern, src[i], dst[i]);
			g_free ( build_patern);
		}
	}

	return result;
}


gchar *gw_str_blob_to_regex ( gchar *blob)
{
	gchar *regex = NULL;
	gchar *regex_build = NULL;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( blob != NULL )
	{
		regex = g_strdup ( blob);
		regex_build = gw_str_replace_str ( regex, "\\", "\\\\");
		g_free ( regex);
		regex = regex_build;
		regex_build = gw_str_replace_str ( regex, ".", "\\.");
		g_free ( regex);
		regex = regex_build;
		regex_build = gw_str_replace_str ( regex, "*", ".*");
		g_free ( regex);
		regex = regex_build;
		regex_build = gw_str_replace_str ( regex, "?", ".");
		g_free ( regex);
		regex = regex_build;
		regex_build = gw_str_replace_str ( regex, "^", "\\^");
		g_free ( regex);
		regex = regex_build;
		regex_build = gw_str_replace_str ( regex, "$", "\\$");
		g_free ( regex);
		regex = regex_build;
		regex_build = gw_str_replace_str ( regex, "|", "\\|");
		g_free ( regex);
		regex = regex_build;
		regex_build = gw_str_replace_str ( regex, "(", "\\(");
		g_free ( regex);
		regex = regex_build;
		regex_build = gw_str_replace_str ( regex, ")", "\\)");
		g_free ( regex);
		regex = regex_build;
		regex_build = gw_str_replace_str ( regex, "[", "\\[");
		g_free ( regex);
		regex = regex_build;
		regex_build = gw_str_replace_str ( regex, "]", "\\]");
		g_free ( regex);
		regex = regex_build;
		regex_build = gw_str_replace_str ( regex, "{", "\\{");
		g_free ( regex);
		regex = regex_build;
		regex_build = gw_str_replace_str ( regex, "}", "\\}");
		g_free ( regex);
		regex = regex_build;
		regex_build = gw_str_replace_str ( regex, "+", "\\+");
		g_free ( regex);
		regex = regex_build;
		regex_build = gw_str_replace_str ( regex, "?", "\\?");
		g_free ( regex);
		regex = regex_build;
		regex_build = gw_str_replace_str ( regex, ",", "\\,");
		g_free ( regex);
		regex = regex_build;
		regex_build = g_strconcat ( "^", regex, "$", NULL);
		g_free ( regex);
		regex = regex_build;
	}

	return regex;
}


gint gw_strcmp_strblob ( gchar *str, gchar *blob, gboolean case_sensitive)
{
	gchar *tmp_str = NULL;
	gchar *tmp_blob = NULL;
	gchar *keep_str = NULL;
	gchar **words = NULL;
	gchar *tmp = NULL;
	gint i;
	gint result = -1;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( ( str != NULL ) || ( blob != NULL ) )
	{
		tmp_str = keep_str = g_strdup ( str);
		tmp_blob = g_strdup ( blob);

		if ( case_sensitive )
		{
			g_strdown ( tmp_str);
                		g_strdown ( tmp_blob);
		}

		words = g_strsplit ( tmp_blob, "*", 0);

		for ( i = 0; words[i] != NULL; i++)
		{
			if ( (tmp = strstr ( tmp_str, words[i])) != NULL )
			{
				tmp_str = tmp;

				if ( words[i + 1] == NULL )
				{
					if ( tmp_blob[strlen ( tmp_blob) - 1] != '*' )
					{
						if ( strlen ( words[i]) != strlen ( tmp_str))
						{
							tmp_str = NULL;

							break;
						}
					}
				}
			}
			else
			{
				tmp_str = NULL;

				break;
			}
		}

		if ( tmp_str != NULL )
		{
			if ( strcmp ( words[0], "" ) != 0)
			{
				if ( strncmp ( keep_str, words[0], strlen ( words[0])) == 0 )
				{
					result = 0;
				}
			}
			else { if ( strcmp ( words[0], "" ) == 0)
			{
				result = 0;
			}}
		}

		g_free ( keep_str);
		g_free ( tmp_blob);
		g_strfreev ( words);
	}

#ifdef GW_DEBUG_TOOLS_COMPONENT
	if ( result == 0 )
	{
		g_print ( "*** GW - %s (%d) :: %s() : %s == %s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, str, blob);
	}
	else
	{
		g_print ( "*** GW - %s (%d) :: %s() : %s <> %s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, str, blob);
	}
#endif

	return result;
}


gint gw_strcmp_strregex ( gchar *str, gchar *regex, gboolean case_sensitive)
{
        regex_t *r;
        gint cflags;
        size_t no_sub;
        regmatch_t *reg_result;
        gint result = -1;


#ifdef GW_DEBUG_TOOLS_COMPONENT
        g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

        if ( (r = (regex_t *) g_malloc ( sizeof ( regex_t))) != NULL )
        {
                if ( case_sensitive == FALSE )
                {
                        cflags = REG_ICASE;
                }
                else
                {
                        cflags = 0;
                }

                if ( regcomp ( r, regex, cflags) == 0 )
                {
                        no_sub = r->re_nsub + 1;

                        if ( (reg_result = (regmatch_t *) g_malloc ( sizeof ( regmatch_t) *no_sub)) != NULL )
                        {
                                if ( regexec ( r, str, no_sub, reg_result, 0) == 0 )
                                {
                                        result = 0;
                                }

                                g_free ( reg_result);
                        }
                }

                regfree ( r);
        }

#ifdef GW_DEBUG_TOOLS_COMPONENT
        if ( result == 0 )
        {
                g_print ( "*** GW - %s (%d) :: %s() : %s == %s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, str, regex);
        }
        else
        {
                g_print ( "*** GW - %s (%d) :: %s() : %s <> %s\n", __FILE__, __LINE__, __PRETTY_FUNCTION__, str, regex);
        }
#endif

        return result;
}


gint gw_str_trim_doubled_char ( gchar *str)
{
	char *ptrcar, *ptrsearch, *ptrsrc, *ptrdest;
	gint result = -1;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( str != NULL )
	{
		ptrcar = str;

		while ( *ptrcar)
		{
			ptrsearch = ptrcar + 1;

			while ( *ptrsearch)
			{
				if ( *ptrsearch == *ptrcar )
				{
					ptrsrc = ptrdest = ptrsearch;
					ptrsrc++;

					while ( *ptrsrc)
					{
						if ( *ptrsrc != *ptrcar )
						{
							*(ptrdest++) = *(ptrsrc++);
						}
						else
						{
							ptrsrc++;
						}
					}

					*ptrdest = '\0';
				}

				ptrsearch++;
			}

			ptrcar++;
		}

		result = 0;
	}

	return result;
}


gint gw_str_trim ( gchar* str)
{
	unsigned long nb_space, size;
	gint result = -1;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( str != NULL )
	{
		// Gets the string size
		size = strlen ( str);

		// Counts the number of begining white spaces
		nb_space = 0;

		while ( isspace ( str[nb_space]) && (str[nb_space]))
		{
			nb_space++;
		}

		if ( !str[nb_space])
		{
			// The string is only white spaces. Returns empty string
			str[0] = '\0';
		}
		else
		{
			// Removes all begining white spaces
			if ( nb_space)
			{
				memmove ( str, str + nb_space, size - nb_space);
				size -= nb_space;
			}

			// Removes all ending white spaces
			while ( size && ( isspace ( str[size-1])))
			{
				size--;
			}

			str[size] = '\0';
		}

		result = 0;
	}

	return result;
}


gint gw_str_trim2 ( gchar *str)
{
	gint result = -1;
	gint i = 0, j = 0, end = 0;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( str != NULL )
	{
		/* Counts all spaces. */
		while ( str[i] == ' ')
		{
			i++;
		}

		/* If there are spaces move all characters. */
		if ( i != 0 )
		{
			while ( (str[i] != '\0'))
			{
				str[j] = str[i];
				i++;
				j++;

				if ( str[i] != ' ')
				{
					end = i;
				}
			}
		}
		else
		{
			while ( (str[i] != '\0'))
			{
				if ( str[i] != ' ')
				{
					end = i;
				}
	
				i++;
			}
		}

		if ( end < i )
		{
			for ( j = (end+1); str[j] != '\0'; j++ )
			{
				str[j] = '\0';
			}
		}
		else if ( i != 0 )
		{
			str[j] = '\0';
		}

		result = 0;
	}

	return result;
}


gint gw_str_trim_left ( gchar *str)
{
	gint result = -1;
	gint i = 0, j = 0;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( str != NULL )
	{
		/* Counts all spaces. */
		while ( str[i] == ' ')
		{
			i++;
		}

		/* If there are spaces move all characters. */
		while ( (str[i] != '\0') && (i != 0))
		{
			str[j] = str[i];
			i++;
			j++;
		}

		if ( i != 0 )
		{
			str[j] = '\0';
		}

		result = 0;
	}

	return result;
}


gint gw_str_trim_right ( gchar *str)
{
	gint result = -1;
	gint i = 0;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( str != NULL )
	{
		i = strlen ( str);
	
		/* Counts all spaces. */
		while ( (str[i - 1] == ' ') && ( i != 0))
		{
			str[i - 1] = '\0';
			i--;
		}

		result = 0;
	}

	return result;
}


gint gw_str_delete_char ( gchar *str, gchar c)
{
	gint i = 0, j;
	gint result = -1;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( str != NULL )
	{
		while ( i < strlen ( str))
		{
			if ( str[i] == c )
  			{
  				j = i;

				while ( j < ( strlen ( str)))
				{
					str[j] = str[j + 1];
					j = j + 1;
				}
			}
			else
			{
				i = i + 1;
			}
		}

		result = 0;
	}

	return result;
}


gint gw_str_replace_char ( gchar *string, gchar before, gchar after)
{
	gchar *p = NULL;
	gint result = -1;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( string != NULL )
	{
		for ( p = string; *p; p++)
		{
			if ( *p == before )
			{
				*p = after;
			}
		}

		result = 0;
	}

	return result;
}


gint gw_str_substr ( const gchar *str, gint begin, gint end, gchar *dest)
{
	gint len = 0;
	gint result = -1;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( str != NULL )
	{
		len = end + 1 - begin;
		dest[len] = '\0';
		memcpy ( dest, (gchar *)str + begin, len);

		result = len;
	}

	return result;
}


gint gw_str_dupsubstr ( const gchar *str, gint begin, gint end, gchar **dest)
{
	gint len = 0;
	gint result = -1;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( str != NULL )
	{
		len = end + 1 - begin;

		*dest = (gchar*)g_malloc ( sizeof ( gchar) * (len + 1));

		dest[len] = '\0';
		memcpy ( *dest, (gchar *)str + begin, len);

		result = len;
	}

	return result;
}


gint gw_str_to_lower ( gchar *str)
{
	gint result = -1;
	gint i = 0;


#ifdef GW_DEBUG_TOOLS_COMPONENT
	g_print ( "*** GW - %s (%d) :: %s()\n", __FILE__, __LINE__, __PRETTY_FUNCTION__);
#endif

	if ( str != NULL )
	{
		while ( str[i] != '\0' )
		{
			str[i] = tolower ( str[i]);
			i++;
		}

		result = 0;
	}

	return result;
}


gint gw_str_to_ascii ( gchar *str, gint len)
{
	gint result = -1;
	gint i = 0;


	if ( str != NULL)
	{
		/* Counts all spaces. */
		while ( i < len)
		{
			if ( isascii ( str[i]) == 0)
			{
				str[i] = ' ';
			}
			i++;
		}
		
		result = 0;
	}

	return result;
}


gint gintlen ( const gint n)
{
	gint len = 1;
	gdouble quo = 1;
	gdouble result = 1.0;


	result = abs ( n) / quo;

	while ( result > 1)
	{
		len ++;
		quo = quo * 10;
		result = abs ( n) / quo;
	}

	len--;

	return len;
}
